/// <reference types="node" />
/// <reference types="node-fetch" />

import { Context, ILogger, IResolver, Identity, NextFunction, ObjectId, Resolvable, TypeRegistry } from '@e2/core';
import { FindResult, Index, MongoApplicationSettings, MongoClient, MongoCollection, MongoDatabase, MongoModel } from '@e2/mongodb';
import { HttpError, InternalWebContext, WebApplication, WebApplicationSettings, WebContext, WebController } from '@e2/web';
import { V1Namespace, V1NamespaceList, V1beta1CronJob, V1beta1CronJobList } from '@kubernetes/client-node';
import { Constructor, IAttribute, IInterceptor, IInvocation, TypedConstructor } from 'agentframework';
import { GoogleToken } from 'gtoken';
import { IncomingMessage } from 'http';

export declare class IdentityProfile {
	_id: ObjectId;
	displayName: string;
	displayImageURL: string;
}
export interface IActivatable {
	activated?: boolean;
	activatedAt?: Date;
	activatedFrom?: string;
	activatedBy?: IdentityProfile;
	activatedReason?: string;
}
export interface IArchivable {
	archived?: boolean;
	archivedAt?: Date;
	archivedFrom?: string;
	archivedBy?: IdentityProfile;
	archivedReason?: string;
}
export interface IAuditable {
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
}
export interface IBlockable {
	blocked?: boolean;
	blockedAt?: Date;
	blockedFrom?: string;
	blockedBy?: IdentityProfile;
	blockedReason?: string;
}
export interface IChainable {
	previous?: ObjectId;
}
export interface IChargeable {
	charge?: number;
	chargedAt?: Date;
	chargedFrom?: string;
	chargedBy?: IdentityProfile;
	chargedReason?: string;
	dischargedAt?: Date;
	dischargedFrom?: string;
	dischargedBy?: IdentityProfile;
	dischargedReason?: string;
}
export interface IDeletable {
	deleted?: boolean;
	deletedAt?: Date;
	deletedFrom?: string;
	deletedBy?: IdentityProfile;
	deletedReason?: string;
}
export interface IExpirable {
	expireAt?: Date;
}
export interface IMutable {
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
}
export interface IEdge extends IAuditable, IMutable, IExpirable {
	outType: string;
	label: string;
	inType: string;
	outV: string | any;
	inV: string | any;
	meta: any;
	expireAt: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
}
export interface INameable {
	name: string;
	displayName: string;
}
export interface IProfile {
	displayName: string;
	displayImageURL: string;
}
export interface IStateful {
	state: string;
}
export interface IVerifiable {
	verified?: boolean;
	verifiedAt?: Date;
	verifiedFrom?: string;
	verifiedBy?: IdentityProfile;
	verifiedReason?: string;
	verifyToken?: string;
	verifyTokenFailures?: number;
	verifyTokenExpires?: Date;
}
export interface IVertex extends IAuditable, IMutable, IExpirable {
}
export declare const SYSTEM_MAX_UPLOAD_SIZE: number;
export declare const SYSTEM_MAX_NUMBER_OF_ACCOUNTS_PER_USER = 1;
declare enum OIDC_PROFILE_IMAGE_POLICIES {
	PERSISTED = "persisted",
	CACHED = "cached",
	PROXY = "proxy",
	REDIRECT = "redirect"
}
export interface EmailProvider {
	provider: string;
	domain: Array<string>;
}
export declare class WebFoundationSettings extends WebApplicationSettings implements MongoApplicationSettings {
	ADMIN_EMAIL: string;
	TRUSTED_EMAIL_PROVIDERS: Array<EmailProvider>;
	OWNER_NAME: string;
	OWNER_COMPANY_NAME: string;
	OWNER_COMPANY_CODE: string;
	OWNER_COMPANY_ADDRESS: Array<string>;
	OWNER_COMPANY_TELEPHONE: string;
	OWNER_COMPANY_FAX: string;
	OWNER_AGREEMENTS: Array<any>;
	MONGODB_URL: string;
	MONGODB_DATABASE: string;
	MONGODB_EVENT_LOG_LEVEL: string;
	WEB_SESSION_COOKIE_KEY: string;
	WEB_SESSION_COOKIE_SECRET: Array<string>;
	WEB_SESSION_EXPIRES_IN_MILLISECONDS: number;
	WEB_TOKEN_EXPIRES_IN_MILLISECONDS: number;
	WEB_INVITATION_EXPIRES_IN_MILLISECONDS: number;
	WEB_EMAIL_VERIFICATION_RESEND_COOLDOWN_IN_MILLISECONDS: number;
	WEB_EMAIL_VERIFICATION_EXPIRES_IN_MILLISECONDS: number;
	WEB_SIGNIN_PATH: string;
	WEB_SIGNIN_SUCCESS_PATH: string;
	WEB_SIGNIN_FAILED_PATH: string;
	WEB_ACCESS_DENIED_PATH: string;
	WEB_SIGNOUT_PATH: string;
	WEB_SIGNOUT_COMPLETE_PATH: string;
	MAX_NUMBER_OF_ACCOUNTS_PER_USER: number;
	MAX_UPLOAD_SIZE: number;
	MAX_APFS_CHUNK_SIZE: number;
	OIDC_AUTHENTICATION_EXPIRES_IN_MILLISECONDS: number;
	OIDC_REDIRECT_LIST: Array<string>;
	OIDC_NO_REDIRECTS_LIST: Array<string>;
	OIDC_PROFILE_IMAGE_POLICY: OIDC_PROFILE_IMAGE_POLICIES;
	OIDC_PROFILE_IMAGE_URL: string;
	OIDC_MICROSOFT: boolean;
	OIDC_MICROSOFT_ISSUER_URL: string;
	OIDC_MICROSOFT_CLIENT_ID: string;
	OIDC_MICROSOFT_CLIENT_SECRET: string;
	OIDC_MICROSOFT_SCOPES: Array<string>;
	OIDC_GOOGLE: boolean;
	OIDC_GOOGLE_ISSUER_URL: string;
	OIDC_GOOGLE_CLIENT_ID: string;
	OIDC_GOOGLE_CLIENT_SECRET: string;
	OIDC_GOOGLE_SCOPES: Array<string>;
	OIDC_GITLAB: boolean;
	OIDC_GITLAB_ISSUER_URL: string;
	OIDC_GITLAB_CLIENT_ID: string;
	OIDC_GITLAB_CLIENT_SECRET: string;
	OIDC_GITLAB_SCOPES: Array<string>;
	OIDC_GITHUB: boolean;
	OIDC_GITHUB_ISSUER_URL: string;
	OIDC_GITHUB_CLIENT_ID: string;
	OIDC_GITHUB_CLIENT_SECRET: string;
	OIDC_GITHUB_SCOPES: Array<string>;
	OIDC_WEIBO: boolean;
	OIDC_WEIBO_ISSUER_URL: string;
	OIDC_WEIBO_CLIENT_ID: string;
	OIDC_WEIBO_CLIENT_SECRET: string;
	OIDC_WEIBO_SCOPES: Array<string>;
	CUSTOM_GOOGLE_API_EMAIL_ID: string;
	CUSTOM_GOOGLE_API_PRIVATE_KEY: string;
	CUSTOM_AWS_ACCESS_KEY_ID: string;
	CUSTOM_AWS_SECRET_ACCESS_KEY: string;
	CUSTOM_AWS_REGION: string;
	CUSTOM_STRIPE_PUBLIC_KEY: string;
	CUSTOM_STRIPE_SECRET_KEY: string;
	CUSTOM_MAILGUN_API_KEY: string;
	CUSTOM_KUBERNETES_API_URL: string;
	CUSTOM_KUBERNETES_API_TOKEN: string;
	CUSTOM_AZURE_STORAGE_ACCOUNT: string;
	CUSTOM_AZURE_STORAGE_ACCESS_KEY: string;
}
declare class SecuredCookie extends Resolvable<WebFoundationSettings> {
	private readonly crypts;
	private readonly base72;
	readonly name: string;
	constructor(app: Context<WebFoundationSettings>);
	sign(data: string): string;
	verify(signed: string): string | false;
}
declare class SessionCookieManager extends Resolvable<WebFoundationSettings> {
	security: SecuredCookie;
	readonly name: string;
	get(ctx: WebContext<WebFoundationSettings>): string | null;
	set(ctx: WebContext<WebFoundationSettings>, data: string): void;
	clear(ctx: WebContext<WebFoundationSettings>): void;
}
export interface Session {
	session: ObjectId | null;
	user: ObjectId | null;
	token: ObjectId | null;
	roles: Array<string>;
	groups: Array<string>;
	scope: Array<string>;
	profile: IdentityProfile | null;
	theme: string;
	timezone: string;
	locale: string;
}
declare class InternalSymbol {
	static readonly ServiceContext: unique symbol;
}
export interface FetchConfiguration {
	method: string;
	url: string;
	headers: any;
	data: any;
	responseType: string;
	credentials: boolean;
}
export interface FetchResponse {
	ok: boolean | number;
	headers: any;
	status: number;
	error?: string;
	errcode?: number;
	message?: string;
	stack?: string;
	result?: any;
	results?: Array<any>;
	data?: Buffer;
	stream?: IncomingMessage;
}
export interface FetchParameters {
	url: string | undefined;
	init: FetchConfiguration | undefined;
}
export declare class Fetch extends WebFoundationUserService<WebFoundationSettings> {
	fetch(url: string, init?: FetchConfiguration): Promise<FetchResponse>;
	fetch(init: FetchConfiguration): Promise<FetchResponse>;
	private buildAuthorisedHeaders;
}
export declare class WebIdentity implements Identity {
	id: string;
	address: string;
	session: ObjectId | null;
	user: ObjectId | null;
	roles: Array<string>;
	groups: Array<string>;
	scope: Array<string>;
	token: ObjectId | null;
	profile: IdentityProfile | null;
	theme: string;
	timezone: string;
	locale: string;
}
export interface WebFoundationToken {
}
export interface WebFoundationDomain<T extends WebFoundationSettings> extends ILogger, IResolver {
	readonly identity: WebIdentity;
	readonly token: WebFoundationToken;
	readonly settings: T;
	set(key: string, value: any): any;
	has(key: string): boolean;
	fetch(url: string, init?: Partial<FetchConfiguration>): Promise<FetchResponse>;
	fetch(init: Partial<FetchConfiguration>): Promise<FetchResponse>;
	construct<U1>(type: TypedConstructor<U1>, transit?: boolean): U1;
	resolve<U2>(type: TypedConstructor<U2>, transit?: boolean): Promise<U2>;
	registerSingleton<U3>(type: TypedConstructor<U3>, instance: U3): void;
	releaseSingleton(type: Constructor): void;
	registerType(type: Constructor, initializer: Function, finalizer: Function): void;
	releaseType(type?: Constructor): Array<any>;
	collection<U3>(type: new (db: MongoDatabase, ...args: Array<any>) => U3): U3;
}
export declare abstract class WebFoundationService<T extends WebFoundationSettings> extends Resolvable<T> {
}
export declare abstract class WebFoundationUserService<T extends WebFoundationSettings> extends Resolvable<T> {
	private [InternalSymbol.ServiceContext];
	constructor(app: WebFoundationApplication<T>, ctx: any);
	readonly domain: WebFoundationDomain<T>;
	addResponseHeader(name: string, value: number | string): void;
	getRequestHeader(name: string): string | string[] | undefined;
}
declare abstract class WfService extends WebFoundationService<WebFoundationSettings> {
}
export declare class WebSessionService extends WfService {
	readonly name: string;
	sessionCookie: SessionCookieManager;
	create(ctx: WebContext<WebFoundationSettings>, id?: string): Partial<Session> | null;
	update(ctx: WebContext<WebFoundationSettings>, session: Partial<Session>): void;
	clear(ctx: WebContext<WebFoundationSettings>): void;
}
export declare class WebFoundationApplication<T extends WebFoundationSettings> extends WebApplication<T> {
	WebIdentityService: WebSessionService;
	protected createRequestState(ctx: InternalWebContext<T>, state?: any): any;
	protected createRequestIdentity(ctx: InternalWebContext<T>): Identity;
	protected createRequestSession(ctx: InternalWebContext<T>): Partial<Session> | null;
	onInitApplication(): void;
}
export interface IUserSettings {
	MAX_NUMBER_OF_ACCOUNTS_PER_USER: number;
	MAX_UPLOAD_SIZE: number;
	MAX_STORAGE_SIZE: number;
}
export interface IPostalAddress {
	address: string;
	postalCode: string;
	city: string;
	state: string;
	countryCode: string;
	country: string;
}
export interface IUser {
	id: number;
	name: string;
	displayName: string;
	displayImageURL: string;
	settings: IUserSettings;
	signature?: any;
	location?: any;
	address: IPostalAddress;
	email?: string;
	mobile?: string;
	roles: Array<string>;
	identities: any;
	theme?: string;
	timezone?: string;
	locale?: string;
	password: string;
	passwordSalt: string;
	paymentPassword: string;
	paymentPasswordSalt: string;
}
export declare class Collection<T extends ImmutableModel> extends MongoCollection<T> {
}
export declare class ImmutableModel extends MongoModel implements IAuditable {
	constructor();
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	$collection<T extends ImmutableModel>(): Collection<T>;
}
export declare class Model extends ImmutableModel implements IExpirable, IMutable, IActivatable, IBlockable, IDeletable {
	expireAt: Date;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
	activated?: boolean;
	activatedAt?: Date;
	activatedFrom?: string;
	activatedBy?: IdentityProfile;
	activatedReason?: string;
	blocked?: boolean;
	blockedAt?: Date;
	blockedFrom?: string;
	blockedBy?: IdentityProfile;
	blockedReason?: string;
	deleted?: boolean;
	deletedAt?: Date;
	deletedFrom?: string;
	deletedBy?: IdentityProfile;
	deletedReason?: string;
}
export declare class UserModel extends Model implements IUser {
	id: number;
	name: string;
	displayName: string;
	displayImageURL: string;
	signature?: any;
	location?: any;
	address: IPostalAddress;
	email?: string;
	mobile?: string;
	roles: Array<string>;
	identities: any;
	settings: IUserSettings;
	theme?: string;
	timezone?: string;
	locale?: string;
	password: string;
	passwordSalt: string;
	paymentPassword: string;
	paymentPasswordSalt: string;
}
export interface WebFoundationContextState {
	uid: string;
	user?: UserModel;
	[x: string]: any;
}
export interface WebFoundationContext<T extends WebFoundationSettings> extends WebContext<T> {
	identity: WebIdentity;
	state: WebFoundationContextState;
	domain: WebFoundationDomain<T>;
}
export interface WfCtx extends WebFoundationContext<WebFoundationSettings> {
}
export interface WfDomain extends WebFoundationDomain<WebFoundationSettings> {
}
declare enum EventType {
	SIGN_UP = "sign-up",
	SIGN_IN = "sign-in",
	SIGN_OUT = "sign-out"
}
export declare class EventModel extends Model {
	id: number;
	type: EventType;
	context: any;
}
export interface OpenIdProviderConfigurationMandatoryFields {
	issuer: string;
	authorization_endpoint: string;
	jwks_uri: string;
	response_types_supported: Array<string>;
	subject_types_supported: Array<string>;
	token_endpoint: string;
	id_token_signing_alg_values_supported: Array<string>;
	scopes_supported: Array<string>;
	claims_supported: Array<string>;
}
export interface OpenIdProviderConfigurationOptionalFields {
	userinfo_endpoint?: string;
	registration_endpoint?: string;
	response_modes_supported?: Array<string>;
	grant_types_supported?: Array<string>;
	acr_values_supported?: Array<string>;
	id_token_encryption_alg_values_supported?: Array<string>;
	id_token_encryption_enc_values_supported?: Array<string>;
	userinfo_signing_alg_values_supported?: Array<string>;
	userinfo_encryption_alg_values_supported?: Array<string>;
	userinfo_encryption_enc_values_supported?: Array<string>;
	request_object_signing_alg_values_supported?: Array<string>;
	request_object_encryption_alg_values_supported?: Array<string>;
	request_object_encryption_enc_values_supported?: Array<string>;
	token_endpoint_auth_methods_supported?: Array<string>;
	token_endpoint_auth_signing_alg_values_supported?: Array<string>;
	display_values_supported?: Array<string>;
	claim_types_supported?: Array<string>;
	service_documentation?: string;
	claims_locales_supported?: Array<string>;
	ui_locales_supported?: Array<string>;
	claims_parameter_supported?: boolean;
	request_parameter_supported?: boolean;
	request_uri_parameter_supported?: boolean;
	require_request_uri_registration?: boolean;
	op_policy_uri?: string;
	op_tos_uri?: string;
}
export interface OpenIdProviderConfigurationSessionFields {
	check_session_iframe?: string;
	end_session_endpoint?: string;
}
export interface OpenIdProviderConfiguration extends OpenIdProviderConfigurationMandatoryFields, OpenIdProviderConfigurationOptionalFields, OpenIdProviderConfigurationSessionFields {
}
export declare type ProviderProtocols = string | 'openid' | 'oauth' | 'oauth2' | 'bot';
export interface IExternalProviderPrimaryKey {
	provider: string;
	providerProtocol: ProviderProtocols;
	providerURL: string;
	appID: string;
}
export interface IExternalProviderProfile {
	provider: string;
	providerProtocol: ProviderProtocols;
	providerURL: string;
	providerFingerprint?: string;
	providerConfiguration?: any;
	appID?: string;
	appSecret?: string;
	scope?: string;
	displayName?: string;
	displayImageURL?: string;
	description?: string;
}
export interface IExternalProvider extends IExternalProviderPrimaryKey, IProfile {
	provider: string;
	providerProtocol: ProviderProtocols;
	providerURL: string;
	providerFingerprint?: string;
	providerConfiguration?: any;
	appID: string;
	appSecret: string;
	scope: string;
	displayName: string;
	displayImageURL: string;
	description: string;
}
export declare class ExternalProviderModel extends Model implements IExternalProvider {
	provider: string;
	providerProtocol: ProviderProtocols;
	providerURL: string;
	providerFingerprint: string;
	providerConfiguration: OpenIdProviderConfiguration;
	appID: string;
	appSecret: string;
	scope: string;
	displayName: string;
	displayImageURL: string;
	description: string;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	expireAt: Date;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
}
declare class EventService extends WfService {
	createEvent(domain: WfDomain, type: EventType, context?: any): Promise<EventModel | null>;
	getLatestEvents(domain: WfDomain): Promise<Array<EventModel>>;
	createSignInEvent(domain: WfDomain, idProvider: ExternalProviderModel): Promise<EventModel | null>;
	createSignUpEvent(domain: WfDomain, idProvider: ExternalProviderModel): Promise<EventModel | null>;
	createSignOutEvent(domain: WfDomain, idProvider: ExternalProviderModel): Promise<EventModel | null>;
}
export declare abstract class WebFoundationController<T extends WebFoundationSettings> extends WebController<T> {
	protected Events: EventService;
	protected getFailureURL(ctx: WfCtx, err: Error): Promise<string>;
}
export declare class DatabaseError extends HttpError {
	readonly statusCode: number;
	readonly statusMessage: string;
}
export declare class ValidationError extends HttpError {
	readonly statusCode: number;
	readonly statusMessage: string;
}
export declare function user(): MethodDecorator;
export declare function admin(): MethodDecorator;
export declare class RoleAttribute implements IAttribute, IInterceptor {
	private role;
	constructor(role: string);
	beforeDecorate(target: Object | Function, targetKey?: string | symbol, descriptor?: PropertyDescriptor): boolean;
	getInterceptor(): IInterceptor;
	intercept(target: IInvocation, parameters: ArrayLike<any>): any;
}
export declare class Time {
	static duration(from: Date, to: Date): string;
}
export interface FormatterOptions {
	format: 'time' | 'percentage' | 'byte' | 'currency' | 'number';
	format_currency_code?: string;
	start?: number;
	end?: number;
}
export declare function formatTime(num: number): string;
export declare function formatNumber(num: number, options: FormatterOptions): string;
export declare function merge(...sources: Array<any>): any;
export interface WfApp extends WebFoundationApplication<WebFoundationSettings> {
}
export declare class Database extends MongoDatabase {
	readonly identity: Partial<WebIdentity>;
	constructor(client: MongoClient, app: WfApp, ctx: WfCtx);
	audit(action: string, updates?: object, status?: boolean): object;
}
export interface ModelConstructor<T extends Model> {
	new (): T;
	getType(): string;
}
export declare class Validator {
	static validate(model: Model & IDeletable & IBlockable & IExpirable): void;
}
export declare class VertexModel extends Model implements IVertex {
	findOutVertices<T extends Model>(outCollection: Collection<T>, label?: string): Promise<Array<T>>;
}
export declare class EdgeModel extends Model implements IEdge {
	outType: string;
	label: string;
	inType: string;
	outV: ObjectId | any;
	inV: ObjectId | any;
	meta: any;
}
export declare class EdgeCollection extends Collection<EdgeModel> {
	constructor(db: Database);
	findInVertices<T extends Model>(outV: ObjectId, outType: string, inCollection: Collection<T>, label?: string): Promise<Array<T>>;
	findOutVertices<T extends Model>(inV: ObjectId, inType: string, outCollection: Collection<T>, label?: string): Promise<Array<T>>;
	outV(outV: ObjectId, label: string, filter?: any, projection?: any, sort?: any): Promise<Array<EdgeModel>>;
	inV(inV: ObjectId, label: string, outType: string, filter?: any, projection?: any, sort?: any): Promise<Array<EdgeModel>>;
	removeRelationships(target: ObjectId): Promise<Array<EdgeModel>>;
	static getIndices(): Array<Index>;
	createIndices(): Promise<any>;
}
export declare class EmailModel extends Model {
	name: string;
	displayName: string;
	user: ObjectId;
	verified?: boolean;
	verifiedAt?: Date;
	verifiedBy?: IdentityProfile;
	verifiedFrom?: string;
	verifiedReason?: string;
	verifyToken: string;
	verifyTokenFailures: number;
	verifyTokenExpires: Date;
	resetPasswordAt?: Date;
	resetPasswordFrom?: string;
	resetPasswordBy?: IdentityProfile;
	resetPasswordToken?: string;
	resetPasswordTokenFailures?: number;
	resetPasswordTokenExpires?: Date;
}
export declare class EmailCollection extends Collection<EmailModel> {
	constructor(db: Database);
	static getIndices(): Array<any>;
	createIndices(): Promise<any>;
}
export declare class MobileModel extends Model {
	name: string;
	displayName: string;
	user: ObjectId;
	verified?: boolean;
	verifiedAt?: Date;
	verifiedBy?: IdentityProfile;
	verifiedFrom?: string;
	verifiedReason?: string;
	verifyToken: string;
	verifyTokenFailures: number;
	verifyTokenExpires: Date;
	resetPasswordAt?: Date;
	resetPasswordFrom?: string;
	resetPasswordBy?: IdentityProfile;
	resetPasswordToken?: string;
	resetPasswordTokenFailures?: number;
	resetPasswordTokenExpires?: Date;
}
export declare class MobileCollection extends Collection<MobileModel> {
	constructor(db: Database);
	static getIndices(): Array<any>;
	createIndices(): Promise<any>;
}
export declare class ExternalProviderCollection extends Collection<ExternalProviderModel> {
	constructor(db: Database);
	static getIndices(): Array<Index>;
	createIndices(): Promise<any>;
}
export interface IExternalUserIdentity {
	type: string;
	key?: string;
	value: string;
}
export interface IExternalUserKeyFields {
	uid: number | string;
	providerId: ObjectId;
}
export interface IExternalUserMandatoryFields extends IExternalUserKeyFields {
	uid: number | string;
	provider: string;
	providerProtocol: string;
	providerId: ObjectId;
	providerData: any;
	userData: any;
	identities: Array<IExternalUserIdentity>;
}
export interface IExternalUserOptionalFields {
	user: ObjectId | null;
	username: string | null;
	domain: string | null;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string | null;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
	locale: string | null;
}
export interface IExternalUser extends IVertex, IExternalUserMandatoryFields, IExternalUserOptionalFields, IMutable, IExpirable, IBlockable, IDeletable {
	uid: number | string;
	provider: string;
	providerProtocol: string;
	providerId: ObjectId;
	providerData: any;
	userData: any;
	identities: Array<IExternalUserIdentity>;
	user: ObjectId | null;
	username: string | null;
	domain: string | null;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string | null;
	displayImageFile: ObjectId | null;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
	locale: string | null;
	expireAt: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
	blocked: boolean;
	blockedAt: Date;
	blockedFrom: string;
	blockedBy: IdentityProfile;
	blockedReason: string;
	deleted: boolean;
	deletedAt: Date;
	deletedFrom: string;
	deletedBy: IdentityProfile;
	deletedReason: string;
}
export declare class ExternalUserModel extends Model implements IExternalUser {
	uid: number | string;
	provider: string;
	providerProtocol: string;
	providerId: ObjectId;
	providerData: any;
	userData: any;
	identities: Array<IExternalUserIdentity>;
	user: ObjectId | null;
	username: string;
	domain: string;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string;
	displayImageFile: ObjectId;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
	locale: string | null;
	blocked: boolean;
	blockedAt: Date;
	blockedFrom: string;
	blockedBy: IdentityProfile;
	blockedReason: string;
	deleted: boolean;
	deletedAt: Date;
	deletedFrom: string;
	deletedBy: IdentityProfile;
	deletedReason: string;
}
export declare class ExternalUserCollection extends Collection<ExternalUserModel> {
	constructor(db: Database);
	static getIndices(): Array<Index>;
	createIndices(): Promise<any>;
}
export interface IAuthenticationToken {
	token_type: string;
	scope: string;
	expires_in: number;
	ext_expires_in: number;
	access_token: string;
	refresh_token: string;
	id_token: any | null;
}
export declare class SessionModel extends Model implements IBlockable, IChainable, IDeletable {
	device: string;
	provider: string;
	providerId: ObjectId;
	authenticator: ObjectId;
	authenticationToken: IAuthenticationToken;
	user: ObjectId;
	roles: Array<string>;
	token: ObjectId;
	tokens: Array<any>;
	previous: ObjectId;
	impersonated: boolean;
	impersonatedAt: Date;
	impersonatedFrom: string;
	impersonatedBy: IdentityProfile;
	impersonatedReason: string;
	blocked: boolean;
	blockedAt: Date;
	blockedFrom: string;
	blockedBy: IdentityProfile;
	blockedReason: string;
	deleted: boolean;
	deletedAt: Date;
	deletedFrom: string;
	deletedBy: IdentityProfile;
	deletedReason: string;
}
export declare class SessionCollection extends Collection<SessionModel> {
	constructor(db: Database);
	static getIndices(): Array<Index>;
	createIndices(): Promise<any>;
}
export declare class UserCollection extends Collection<UserModel> {
	constructor(db: Database);
	static getIndices(): Array<any>;
	createIndices(): Promise<any>;
}
export interface ApfsFileHash {
	md5: Buffer;
	sha1: Buffer;
	sha256: Buffer;
	ipfs?: string;
}
export interface ApfsFileAccessToken {
	token: string;
	description?: string;
	notValidBefore: Date;
	notValidAfter?: Date;
	downloadQuota?: number;
	download: number;
}
export interface ApfsFileAccessStatistics {
	update: number;
	read: number;
}
export interface ApfsFileChunkInfo {
	_id: ObjectId;
	offset: number;
	byteLength: number;
}
export interface ApfsFileMetadata {
	originalname?: string;
	width?: number;
	height?: number;
	duration?: number;
}
export interface ApfsFileLike extends ApfsFileMetadata {
	data: Buffer;
	type: string;
	filename: string;
}
export declare class ApfsFile implements ApfsFileMetadata {
	filename: string;
	type: string;
	data: Buffer;
	byteLength: number;
	hash: ApfsFileHash;
	createdAt: Date;
	updatedAt?: Date;
	context: any;
	stats: ApfsFileAccessStatistics;
	originalname: string;
	width: number;
	height: number;
	duration: number;
}
export declare class ApfsFileChunkModel extends ImmutableModel {
	hash: ApfsFileHash;
	type: string;
	byteLength: number;
	data: Buffer;
	dataOffset: number;
	dataByteLength: number;
	dataEncoding: 'binary';
	expireAt: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
}
export declare class ApfsFileChunkCollection extends Collection<ApfsFileChunkModel> {
	constructor(db: Database);
	static getIndices(): Array<Index>;
	createIndices(): Promise<any>;
}
export declare class ApfsFileModel extends Model implements IChainable {
	toUrl(): string;
	path: string;
	name: string;
	type: string;
	byteLength: number;
	originalname: string;
	width: number;
	height: number;
	duration: number;
	hash: ApfsFileHash;
	chunks: Array<ApfsFileChunkInfo>;
	context: any;
	stats: ApfsFileAccessStatistics;
	tokens: Array<ApfsFileAccessToken>;
	previous: ObjectId;
}
export declare class ApfsFileCollection extends Collection<ApfsFileModel> {
	constructor(db: MongoDatabase);
	static getIndices(): Array<Index>;
	createIndices(): Promise<any>;
}
export declare class AdminService extends WebFoundationService<WebFoundationSettings> {
	constructor(app: WebFoundationApplication<WebFoundationSettings>);
	createIndices(domain: WfDomain): Promise<Array<any>>;
}
export declare class ApfsService extends Resolvable<WebFoundationSettings> {
	readonly name: string;
	parse(url: string): Partial<ApfsFileLike> | null;
	download(domain: WfDomain, url: string, overwrite?: boolean): Promise<ApfsFileModel | null>;
	save(domain: WfDomain, apfs: ApfsFileLike, overwrite?: boolean): Promise<ApfsFileModel>;
	load(domain: WfDomain, url: string): Promise<ApfsFile | null>;
}
export interface UserCredential {
	username: string;
	password: string;
}
export declare class UserService extends WfService {
	findOneById(domain: WfDomain, id: string | ObjectId | null | undefined): Promise<UserModel | null>;
	findOneByName(domain: WfDomain, name: string): Promise<UserModel | null>;
	findOneByEmail(domain: WfDomain, emailAddress: string | null | undefined): Promise<UserModel | null>;
	insertOne(domain: WfDomain, user: IUser): Promise<UserModel>;
	update(domain: WfDomain, user: ObjectId | string | null, name: string, value: any): Promise<UserModel | null>;
	updatePassword(domain: WfDomain, userId: ObjectId | string | null, password: string): Promise<UserModel | null>;
	getCredential(domain: WfDomain, userId: ObjectId | string | null): Promise<UserCredential | null>;
}
export declare class UserMutations {
	static insertOne(domain: WfDomain, user: Partial<IUser>): Promise<UserModel>;
	static updateUser(domain: WfDomain, user: ObjectId | string, name: string, value: any): Promise<UserModel | null>;
	static updateUserFields(domain: WfDomain, user: ObjectId | string, fields: any): Promise<UserModel | null>;
}
export declare class CurrentUser extends UserModel {
	static [TypeRegistry.Initializer](app: WfApp, ctx: WfCtx): Promise<CurrentUser | null>;
}
export declare class UserGetters {
	static findOneById(domain: WfDomain, id: any): Promise<UserModel | null>;
	static findOneByName(domain: WfDomain, name: string): Promise<UserModel | null>;
}
export declare class EmailGetters {
	static findOneByEmail(domain: WfDomain, emailAddress: string): Promise<EmailModel | null>;
	static findOneByUser(domain: WfDomain, user: ObjectId | string): Promise<EmailModel | null>;
}
export declare class MobileGetters {
	static findOneByEmail(domain: WfDomain, mobile: string): Promise<MobileModel | null>;
	static findOneByUser(domain: WfDomain, user: ObjectId | string): Promise<MobileModel | null>;
}
export interface IAuthenticator {
	_id: ObjectId;
	provider: string;
	providerId: ObjectId;
	displayName: string;
	displayImageURL: string;
	token: any;
}
export declare class SessionMutations {
	static CreateSession(domain: WfDomain, user: UserModel, authenticator: IAuthenticator, allows: Array<string>, context?: any, previousSession?: SessionModel | null | undefined, impersonated?: boolean | undefined): Promise<SessionModel>;
	static CreateOrUpdateSession(domain: WfDomain, user: UserModel, allows: Array<string>, account: string, context?: any): Promise<SessionModel | null>;
	static ExtendSession(domain: WfDomain, context: any): Promise<SessionModel>;
	static RestoreSession(domain: WfDomain, sessionToRestore: SessionModel, context: any, reason: string): Promise<SessionModel | null>;
	static TerminateSession(domain: WfDomain, reason: string): Promise<SessionModel | null>;
}
export declare class WebAuthenticationService extends WfService {
	protected WebSessions: WebSessionService;
	protected Users: UserService;
	login(ctx: WfCtx, user: UserModel | ObjectId | string | null, authenticator: IAuthenticator, allows?: Array<string>): Promise<SessionModel>;
	update(ctx: WfCtx, user: UserModel | ObjectId | string | null): Promise<SessionModel>;
	refresh(ctx: WfCtx, user: UserModel | ObjectId | string | null): Promise<SessionModel>;
	logout(ctx: WfCtx, reason?: string): Promise<SessionModel | null>;
	private updateWebSession;
}
export declare class SessionGetters {
	static GetExternalUserFromUser(domain: WfDomain, user: UserModel, provider: string, providerProtocol: string): Promise<ExternalUserModel | null>;
}
export declare abstract class AuthenticatedController<T extends WebFoundationSettings> extends WebFoundationController<T> {
	__WebFoundationController_InitRequest(ctx: WfCtx, next: NextFunction): Promise<any>;
	protected getFailureURL(ctx: WfCtx, err: Error): Promise<string>;
}
export declare class CurrentSession extends SessionModel {
	static [TypeRegistry.Initializer](app: WebApplication<WebFoundationSettings>, ctx: WfCtx): Promise<SessionModel | null> | null;
}
export declare class ExternalProviderService extends WfService {
	findOrDiscoverExternalProvider(domain: WfDomain, profile: IExternalProviderProfile): Promise<ExternalProviderModel>;
	findByName(domain: WfDomain, name: string): Promise<ExternalProviderModel | null>;
}
export declare class ExternalProviderMutations {
	static findOrDiscoverExternalProvider(domain: WfDomain, profile: IExternalProviderProfile): Promise<ExternalProviderModel>;
	static findExternalProvider(domain: WfDomain, profile: IExternalProviderProfile): Promise<ExternalProviderModel | null>;
	static discoverExternalProvider(domain: WfDomain, profile: IExternalProviderProfile): Promise<ExternalProviderModel>;
	static GetExternalProvider(domain: WfDomain, id: ObjectId | string): Promise<ExternalProviderModel | null>;
	private static discoverIdProviderConfiguration;
}
export interface IAuthenticationProfile extends IExternalUserMandatoryFields, IExternalUserOptionalFields {
	uid: number | string;
	provider: string;
	providerProtocol: string;
	providerId: ObjectId;
	providerData: any;
	userData: any;
	identities: Array<IExternalUserIdentity>;
	username: string | null;
	domain: string | null;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string | null;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
	locale: string | null;
}
export declare class ExternalUserMutations {
	static CreateOrUpdateExternalUser(domain: WfDomain, profile: Partial<IAuthenticationProfile>): Promise<ExternalUserModel>;
	static FindOrCreateUser(domain: WfDomain, externalUser: ExternalUserModel): Promise<UserModel>;
	static CreateNewUserFromExternalUser(domain: WfDomain, externalUser: ExternalUserModel): Promise<UserModel>;
	static UpdateUserFromExternalUser(domain: WfDomain, externalUser: ExternalUserModel): Promise<UserModel>;
	static CreateEmail(domain: WfDomain, externalUser: ExternalUserModel): Promise<EmailModel | null>;
}
export declare class ExternalUserService extends Resolvable<WebFoundationSettings> {
	apfs: ApfsService;
	readonly name: string;
	createOrUpdateExternalUser(domain: WfDomain, profile: Partial<IAuthenticationProfile> | null | undefined): Promise<ExternalUserModel>;
	findOrCreateUser(domain: WfDomain, externalUser: ExternalUserModel): Promise<UserModel>;
	updateUserFromExternalUser(domain: WfDomain, externalUser: ExternalUserModel): Promise<UserModel>;
	saveProfileImage(domain: WfDomain, provider: IExternalProvider, externalUser: ExternalUserModel, type: string, data: Buffer): Promise<string | null>;
	getProfileImage(domain: WfDomain, externalUser: ExternalUserModel): Promise<ApfsFileLike | string | null>;
}
export declare class ExternalUserGetters {
}
export declare class GitLabService extends WfService {
	getProfile(domain: WfDomain, token: any, provider: IExternalProvider): Promise<Partial<IAuthenticationProfile> | null>;
}
export declare class GitHubService extends WebFoundationService<WebFoundationSettings> {
	getProfile(domain: WebFoundationDomain<WebFoundationSettings>, token: any, provider: IExternalProvider): Promise<Partial<IAuthenticationProfile> | null>;
}
export interface GoogleIdToken {
	azp: string;
	aud: string;
	sub: string;
	email: string;
	email_verified: boolean;
	at_hash: string;
	exp: number;
	iss: string;
	iat: number;
	name: string;
	picture: string;
	given_name: string;
	family_name: string;
	locale: string;
}
export interface GoogleToken extends IAuthenticationToken {
	id_token: GoogleIdToken;
}
export declare class GoogleService extends WfService {
	TrustedEmailDomains: string[];
	getProfile(domain: WfDomain, token: GoogleToken, provider: IExternalProvider): Promise<Partial<IAuthenticationProfile> | null>;
}
export interface MicrosoftIdToken {
	ver: string;
	iss: string;
	sub: string;
	aud: string;
	exp: number;
	iat: number;
	nbf: number;
	name: string;
	preferred_username: string;
	oid: string;
	email: string;
	tid: string;
	aio: string;
}
export interface MicrosoftToken extends IAuthenticationToken {
	id_token: MicrosoftIdToken;
}
export declare class MicrosoftService extends WfService {
	apfs: ApfsService;
	TrustedEmailDomains: string[];
	getProfile(domain: WfDomain, token: MicrosoftToken, provider: IExternalProvider): Promise<Partial<IAuthenticationProfile> | null>;
}
export declare class WeiboService extends WfService {
	getProfile(domain: WfDomain, token: any, provider: IExternalProvider): Promise<Partial<IAuthenticationProfile> | null>;
}
export declare abstract class OpenIdController<T extends WebFoundationSettings> extends WebFoundationController<T> {
	protected ExternalUsers: ExternalUserService;
	protected Users: UserService;
	protected abstract discover(ctx: WebFoundationContext<WebFoundationSettings>): Promise<ExternalProviderModel>;
	private getAuthenticator;
	protected OnProfile(ctx: WebFoundationContext<WebFoundationSettings>, token: IAuthenticationToken, provider: ExternalProviderModel): Promise<Partial<IAuthenticationProfile> | null>;
	protected abstract OnAuthenticated(ctx: WebFoundationContext<WebFoundationSettings>, externalProvider: ExternalProviderModel, token: IAuthenticationToken, externalUser: ExternalUserModel, foundUsers: UserModel | null, loggedInUser?: UserModel | null): Promise<UserModel>;
}
export declare abstract class OpenIdAuthController<T extends WebFoundationSettings> extends OpenIdController<T> {
	WebAuth: WebAuthenticationService;
	OnAuthenticated(ctx: WebFoundationContext<WebFoundationSettings>, externalProvider: ExternalProviderModel, token: IAuthenticationToken, externalUser: ExternalUserModel, foundUser: UserModel | null, loggedInUser: UserModel | null): Promise<UserModel>;
}
export declare class GoogleOpenIdAuthController extends OpenIdAuthController<WebFoundationSettings> {
	idp: ExternalProviderService;
	google: GoogleService;
	readonly name: string;
	discover(ctx: WebFoundationContext<WebFoundationSettings>): Promise<ExternalProviderModel>;
	OnProfile(ctx: WebFoundationContext<WebFoundationSettings>, token: IAuthenticationToken, provider: ExternalProviderModel): Promise<Partial<IAuthenticationProfile> | null>;
}
export declare class MicrosoftOpenIdAuthController extends OpenIdAuthController<WebFoundationSettings> {
	idp: ExternalProviderService;
	microsoft: MicrosoftService;
	readonly name: string;
	discover(ctx: WebFoundationContext<WebFoundationSettings>): Promise<ExternalProviderModel>;
	OnProfile(ctx: WebFoundationContext<WebFoundationSettings>, token: IAuthenticationToken, provider: ExternalProviderModel): Promise<Partial<IAuthenticationProfile> | null>;
}
export declare class GitLabOpenIdAuthController extends OpenIdAuthController<WebFoundationSettings> {
	idp: ExternalProviderService;
	gitlab: GitLabService;
	readonly name: string;
	discover(ctx: WebFoundationContext<WebFoundationSettings>): Promise<ExternalProviderModel>;
	OnProfile(ctx: WebFoundationContext<WebFoundationSettings>, token: IAuthenticationToken, provider: ExternalProviderModel): Promise<Partial<IAuthenticationProfile> | null>;
}
export declare class GitHubOpenIdAuthController extends OpenIdAuthController<WebFoundationSettings> {
	idp: ExternalProviderService;
	github: GitHubService;
	readonly name: string;
	discover(ctx: WebFoundationContext<WebFoundationSettings>): Promise<ExternalProviderModel>;
	OnProfile(ctx: WebFoundationContext<WebFoundationSettings>, token: IAuthenticationToken, provider: ExternalProviderModel): Promise<Partial<IAuthenticationProfile> | null>;
}
export declare class WeiboOpenIdAuthController extends OpenIdAuthController<WebFoundationSettings> {
	idp: ExternalProviderService;
	weibo: WeiboService;
	readonly name: string;
	discover(ctx: WebFoundationContext<WebFoundationSettings>): Promise<ExternalProviderModel>;
	OnProfile(ctx: WebFoundationContext<WebFoundationSettings>, token: IAuthenticationToken, provider: ExternalProviderModel): Promise<Partial<IAuthenticationProfile> | null>;
}
export interface IAuthenticator {
	idProvider: ExternalProviderModel;
	getAuthorizeURL(redirect_uri: string, state: any): string;
	getToken(redirect_uri: string, code: string): Promise<IAuthenticationToken>;
	refreshToken(token: IAuthenticationToken): Promise<any>;
	getProfile(token: IAuthenticationToken): Promise<Partial<IAuthenticationProfile> | null>;
	getFailureURL(err: Error): string;
	getSuccessURL(data: any): string;
}
export declare abstract class OrganizationController<T extends WebFoundationSettings> extends WebFoundationController<T> {
}
export interface IOrganizationSettings {
	LICENSE_TYPE: string;
	LICENSE_EXPIRE_AT: Date | null;
	MAX_MEMBERS: number;
}
export interface IOrganization {
	id: number;
	name: string;
	displayName: string;
	displayImageURL: string;
	settings: IOrganizationSettings;
	owner: ObjectId;
}
export declare class OrganizationModel extends VertexModel implements IOrganization, IVerifiable {
	id: number;
	name: string;
	owner: ObjectId;
	displayName: string;
	displayImageURL: string;
	settings: IOrganizationSettings;
	verified?: boolean;
	verifiedAt?: Date;
	verifiedFrom?: string;
	verifiedBy?: IdentityProfile;
	verifiedReason?: string;
	verifyToken?: string;
	verifyTokenFailures?: number;
	verifyTokenExpires?: Date;
}
export declare class OrganizationCollection extends Collection<OrganizationModel> {
	constructor(db: Database);
	static getIndices(): Array<Index>;
	createIndices(): Promise<any>;
	makeIdQuery(account: ObjectId | string | number): any;
}
export declare class OrganizationService extends WfService {
	createOrganization(domain: WfDomain, organization: Partial<IOrganization>, owner?: ObjectId): Promise<OrganizationModel | null>;
	updateOrganization(domain: WfDomain, organizationId: ObjectId | string | number, key: string, field: any): Promise<OrganizationModel | null>;
	findOrganization(domain: WfDomain, organizationId: ObjectId | string | number): Promise<OrganizationModel | null>;
	findOrganizationsByMember(domain: WfDomain, owner: ObjectId): Promise<FindResult<OrganizationModel>>;
	findOrganizationsByOwner(domain: WfDomain, owner: ObjectId): Promise<FindResult<OrganizationModel>>;
	searchOrganization(domain: WfDomain, text: string): Promise<FindResult<OrganizationModel>>;
}
export declare class StripeService extends WfService {
	private readonly client;
	addCard(domain: WfDomain, user: ObjectId | string | null, token: any): Promise<any>;
	listCards(domain: WfDomain, user: ObjectId | string | null): Promise<Array<any>>;
	removeCard(domain: WfDomain, user: ObjectId | string | null, cardId: string): Promise<any>;
	activeCard(domain: WfDomain, user: ObjectId | string | null, cardId: string): Promise<any>;
	setPrimaryCard(domain: WfDomain, user: ObjectId | string | null, card: string): Promise<any>;
	private getCustomer;
	private getStripeCustomer;
	private normalizeCard;
}
export declare class GoogleStorageService extends WfService {
	google: GoogleToken;
	constructor(app: WfApp);
	insert(bucket: string, filename: string, content: any): Promise<any>;
	copy(sourceBucket: string, sourceObject: string, destinationBucket: string, destinationObject: string): Promise<any>;
	remove(bucket: string, filename: string): Promise<any>;
	list(bucket: string, path: string): Promise<any>;
}
export declare class TableService extends WfService {
}
export declare class QueueService extends WfService {
}
export declare class BlobService extends WfService {
}
export interface CronJob {
	cron: string;
	intervalInSeconds: number;
}
export declare class CronJobModel extends Model implements CronJob {
	cron: string;
	intervalInSeconds: number;
	nextScheduleAt: Date;
	lastScheduleAt: Date;
	lastScheduleBy: IdentityProfile;
	lastScheduleFrom: string;
	lastScheduleRetries: number;
	lastScheduleDuration: number;
	lastScheduleResult: string;
	payload: any;
}
export declare class CronJobService extends WebFoundationService<WebFoundationSettings> {
	findAll(domain: WfDomain): Promise<Array<CronJobModel>>;
	insert(domain: WfDomain, cron: string, job: any): Promise<CronJobModel>;
	insert(domain: WfDomain, intervalInSeconds: number, job: any): Promise<CronJobModel>;
	remove(domain: WfDomain, id: ObjectId | string): Promise<CronJobModel>;
	findOneScheduledCronJob(domain: WfDomain): Promise<CronJobModel | null>;
	updateOneScheduledCronJob(domain: WfDomain, id: ObjectId | string, result?: Partial<CronJobModel>): Promise<CronJobModel | null>;
	private calculateNextScheduleTime;
	private calculateNextCronTime;
	private calculateNextIntervalTime;
}
export declare class CronJobCollection extends Collection<CronJobModel> {
	constructor(db: Database);
	static getIndices(): Array<any>;
	createIndices(): Promise<any>;
}
export interface RestOptions {
	method?: string;
	url?: string;
	headers?: any;
	query?: any;
	responseType?: string;
	data?: any;
}
export interface RestResponse {
	status: number;
	headers: any;
	data: any;
}
export declare class RestClient {
	private defaults;
	constructor(defaults?: RestOptions);
	private makeAxiosOptions;
	request(options?: RestOptions): Promise<RestResponse>;
	post(url: string, data?: any, options?: RestOptions): Promise<any>;
	postBuffer(url: string, data: any, options?: RestOptions): Promise<Buffer>;
	put(url: string, data: any, options?: RestOptions): Promise<any>;
	putBuffer(url: string, data: any, options?: RestOptions): Promise<Buffer>;
	get(url: string, options?: RestOptions): Promise<any>;
	getBuffer(url: string, options?: RestOptions): Promise<Buffer>;
	delete(url: string, options?: RestOptions): Promise<any>;
	deleteBuffer(url: string, options?: RestOptions): Promise<Buffer>;
}
export declare class DockerClient extends RestClient {
}
export declare class GitLabClient extends RestClient {
}
export interface GoogleUser {
	id: string;
	businessPhones: Array<string>;
	displayName: string;
	givenName: string;
	jobTitle: string;
	mail: string;
	mobilePhone: string;
	officeLocation: string;
	preferredLanguage: string;
	surname: string;
	userPrincipalName: string;
}
export declare class GoogleClient extends RestClient {
	constructor(access_token: string);
	profile(): Promise<GoogleUser>;
	people(): Promise<any>;
}
export declare class KubernetesClient extends RestClient {
	private api_server;
	constructor(api_server: string, token: string);
	private url;
	listNamespaces(): Promise<V1NamespaceList>;
	getNamespace(name: string): Promise<V1Namespace | null>;
	createNamespace(name: string): Promise<V1Namespace>;
	deleteNamespace(name: string): Promise<any>;
	listCronJobs(namespace?: string): Promise<V1beta1CronJobList>;
	getCronJob(namespace: string, name: string): Promise<V1beta1CronJob>;
	createCronJob(namespace: string, cronjob: any): Promise<any>;
	deleteCronJob(namespace: string, name: string): Promise<any>;
}
export interface MicrosoftUser {
	'@odata.context': string;
	id: string;
	businessPhones: Array<string>;
	displayName: string;
	givenName: string;
	jobTitle: string;
	mail: string;
	mobilePhone: string;
	officeLocation: string;
	preferredLanguage: string;
	surname: string;
	userPrincipalName: string;
}
export interface MicrosoftPhotoMetadata {
	'@odata.context': string;
	'@odata.mediaContentType': string;
	'@odata.mediaEtag': string;
	id: string;
	height: number;
	width: number;
}
export declare class MicrosoftClient extends RestClient {
	constructor(access_token: string);
	profile(): Promise<MicrosoftUser>;
	photo(): Promise<MicrosoftPhotoMetadata | null>;
	photoImage(photoSize?: string): Promise<Buffer>;
	photoImageFile(id: string): Promise<ApfsFileLike | null>;
	photoImageDataURL(): Promise<string | null>;
	users(): Promise<Array<any>>;
}
export declare class OAuthClient extends RestClient {
	constructor(access_token: string);
}
export declare class RedmineClient extends RestClient {
}
export declare class TeamCityClient extends RestClient {
}
export declare class WeiboClient extends RestClient {
	constructor(access_token: string);
	comments(options?: any): Promise<any>;
	mentions(options?: any): Promise<any>;
	reposts(options?: any): Promise<any>;
	weibo(options?: any): Promise<any>;
	getLimits(): Promise<any>;
}
export declare class ExternalProviderGetters {
	static findByName(domain: WfDomain, name: string): Promise<ExternalProviderModel | null>;
	static findById(domain: WfDomain, id: ObjectId | string): Promise<ExternalProviderModel | null>;
}
export interface FetchOptions {
	method?: string;
	url?: string;
	headers?: Headers;
	query?: any;
	data?: any;
}
export interface FetchResponse {
	url: string;
	status: number;
	statusText: string;
	type: string;
	headers: Headers;
	body: Body;
	data: any;
	text: string;
}
declare class FetchClient {
	private defaults;
	constructor(defaults?: FetchOptions);
	private makeFetchOptions;
	request(options: FetchOptions): Promise<FetchResponse>;
	post(url: string, data: any, options?: FetchOptions): Promise<FetchResponse>;
	put(url: string, data: any, options?: FetchOptions): Promise<FetchResponse>;
	get(url: string, options?: FetchOptions): Promise<FetchResponse>;
	delete(url: string, options?: FetchOptions): Promise<FetchResponse>;
}
declare class TableStorageRestClient extends FetchClient {
	protected account: string;
	protected key: string;
	protected endpointSuffix: string;
	protected timeoutInSeconds: number;
	protected version: string;
	constructor(account: string, key: string);
	request(options: FetchOptions): Promise<FetchResponse>;
	post(url: string, data: any, options?: FetchOptions): Promise<FetchResponse>;
	put(url: string, data: any, options?: FetchOptions): Promise<FetchResponse>;
	merge(url: string, data: any, options?: FetchOptions): Promise<FetchResponse>;
	get(url: string, options?: FetchOptions): Promise<FetchResponse>;
	delete(url: string, options?: FetchOptions): Promise<FetchResponse>;
	makeAuthorizationHeader(name: string, resource: string): any;
	makeRequestUrl(name: string, resource?: string): string;
	protected encodeSpecialCharacters(path: string): string;
	protected sign(text: string): string;
	private throwErrorForNon2xxCode;
}
export interface TableEntity {
	PartitionKey: string;
	RowKey: string;
	[x: string]: any;
}
export interface TableEntityResponse {
	PartitionKey: string;
	RowKey: string;
	Timestamp: Date;
	[x: string]: any;
}
export interface ChildTableEntity {
	id: string;
	parentId: string;
}
export declare class TableStorage extends TableStorageRestClient {
	etags: WeakMap<any, string>;
	tableNameRegex: RegExp;
	entityNameRegex: RegExp;
	static [TypeRegistry.Initializer](app: WebFoundationApplication<WebFoundationSettings>, ctx: WebFoundationContext<WebFoundationSettings>): TableStorage;
	queryTables(): Promise<Array<string>>;
	createTable(name: string): Promise<string | null>;
	deleteTable(name: string): Promise<string>;
	batch(tableName: string): TableBatchRestClient;
	insertEntity(tableName: string, entity: TableEntity): Promise<TableEntityResponse>;
	upsertEntity(tableName: string, entity: TableEntity): Promise<TableEntity>;
	updateEntity(tableName: string, primaryKey: string, secondaryKey: string, entity: Partial<TableEntity>): Promise<TableEntity>;
	deleteEntity(tableName: string, primaryKey: string, secondaryKey: string, etag?: string): Promise<boolean>;
	queryEntity<T extends TableEntity>(tableName: string, primaryKey: string, secondaryKey: string): Promise<T | null>;
	queryEntities<T extends TableEntity>(tableName: string, query?: any): Promise<Array<T>>;
	queryEntitiesByPartitionKey(tableName: string, primaryKey: string): Promise<Array<TableEntity>>;
	queryEntitiesWithPrefix(tableName: string, primaryKeyPrefix: string, secondaryKeyPrefix: string): Promise<Array<TableEntity>>;
	getETag(o: any): string | undefined;
	isValidTableName(name: string): boolean;
	isValidTableEntityName(name: string): boolean;
	private fromEntity;
	private toEntity;
}
declare class TableBatchResult {
	ok: number;
	n: number;
}
declare class TableBatchRestClient {
	private client;
	private tableName;
	private readonly batchBoundary;
	private readonly changesetBoundary;
	private primaryKey;
	private secondaryKeys;
	private count;
	private payload;
	constructor(client: TableStorage, tableName: string);
	readonly size: number;
	insertEntity(entity: TableEntity): TableBatchRestClient;
	upsertEntity(entity: TableEntity): TableBatchRestClient;
	deleteEntity(primaryKey: string, secondaryKey: string, etag?: string): TableBatchRestClient;
	updateEntity(primaryKey: string, secondaryKey: string, entity: Partial<TableEntity>): TableBatchRestClient;
	commit(): Promise<TableBatchResult>;
	private validate;
	private addNewRequest;
}